module chlo
using ...IR
import ...IR: NamedAttribute, Value, Location, Block, Region, Attribute, create_operation, context, IndexType
import ..Dialects: namedattribute, operandsegmentsizes
import ...API


"""
`acos`

Returns `Acos(operand)` element-wise.

\$\$
\\acos(x) = 2 * \\atan(\\sqrt(1 - x^2) / (1 + x)) if x != -1
         = pi                                  if x == -1
\$\$
"""
function acos(operand::Value; result=nothing::Union{Nothing, IR.Type}, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[operand, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result) && push!(op_ty_results, result)
    
    create_operation(
        "chlo.acos", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`acosh`

Returns `Acosh(operand)` element-wise.

\$\$
\\acosh(x) = log(x + sqrt(x^2 - 1))      if x >= -1
\\acosh(x) = nan                         if x < -1
\$\$
"""
function acosh(operand::Value; result=nothing::Union{Nothing, IR.Type}, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[operand, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result) && push!(op_ty_results, result)
    
    create_operation(
        "chlo.acosh", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`asin`

Returns `Asin(operand)` element-wise.

\$\$
\\asin(x) = 2 * atan(x / (1 + sqrt(1 - x^2)))
\$\$
"""
function asin(operand::Value; result=nothing::Union{Nothing, IR.Type}, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[operand, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result) && push!(op_ty_results, result)
    
    create_operation(
        "chlo.asin", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`asinh`

Returns `Asinh(operand)` element-wise.

\$\$
\\asinh(x) = log(x + sqrt(x^2 + 1))
\$\$
"""
function asinh(operand::Value; result=nothing::Union{Nothing, IR.Type}, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[operand, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result) && push!(op_ty_results, result)
    
    create_operation(
        "chlo.asinh", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`atan`

Returns `Atan(operand)` element-wise.

\$\$
\\atan(x) = \\atan2(x, 1)
\$\$
"""
function atan(operand::Value; result=nothing::Union{Nothing, IR.Type}, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[operand, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result) && push!(op_ty_results, result)
    
    create_operation(
        "chlo.atan", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`atanh`

Returns `Atanh(operand)` element-wise.

\$\$
\\atanh(x) = 0.5 * log((1 + x) / (1 - x)) if abs(x) <= 1
          = nan                          otherwise
\$\$
"""
function atanh(operand::Value; result=nothing::Union{Nothing, IR.Type}, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[operand, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result) && push!(op_ty_results, result)
    
    create_operation(
        "chlo.atanh", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`bessel_i1e`

Returns `bessel_i1e(operand)` element-wise.
"""
function bessel_i1e(operand::Value; result=nothing::Union{Nothing, IR.Type}, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[operand, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result) && push!(op_ty_results, result)
    
    create_operation(
        "chlo.bessel_i1e", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`broadcast_add`

Returns `lhs + rhs` element-wise.

See
https://www.tensorflow.org/xla/operation_semantics#element-wise_binary_arithmetic_operations.
"""
function broadcast_add(lhs::Value, rhs::Value; result_0=nothing::Union{Nothing, IR.Type}, broadcast_dimensions=nothing, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[lhs, rhs, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result_0) && push!(op_ty_results, result_0)
    !isnothing(broadcast_dimensions) && push!(attributes, namedattribute("broadcast_dimensions", broadcast_dimensions))
    
    create_operation(
        "chlo.broadcast_add", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`broadcast_and`

Returns `logical_and(lhs, rhs)` element-wise.

See
https://www.tensorflow.org/xla/operation_semantics#element-wise_binary_arithmetic_operations.
"""
function broadcast_and(lhs::Value, rhs::Value; result_0=nothing::Union{Nothing, IR.Type}, broadcast_dimensions=nothing, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[lhs, rhs, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result_0) && push!(op_ty_results, result_0)
    !isnothing(broadcast_dimensions) && push!(attributes, namedattribute("broadcast_dimensions", broadcast_dimensions))
    
    create_operation(
        "chlo.broadcast_and", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`broadcast_atan2`

Returns `atan2(lhs/rhs)` element-wise.

See
https://www.tensorflow.org/xla/operation_semantics#element-wise_binary_arithmetic_operations.
"""
function broadcast_atan2(lhs::Value, rhs::Value; result_0=nothing::Union{Nothing, IR.Type}, broadcast_dimensions=nothing, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[lhs, rhs, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result_0) && push!(op_ty_results, result_0)
    !isnothing(broadcast_dimensions) && push!(attributes, namedattribute("broadcast_dimensions", broadcast_dimensions))
    
    create_operation(
        "chlo.broadcast_atan2", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`broadcast_compare`

Compares `lhs` and `rhs` elementwise according to `comparison_direction`
and `compare_type`. If unspecified, `compare_type` is FLOAT for float element
types, SIGNED for signed element types and UNSIGNED for unsigned element
types.

See
https://www.tensorflow.org/xla/operation_semantics#element-wise_comparison_operations.
"""
function broadcast_compare(lhs::Value, rhs::Value; result_0=nothing::Union{Nothing, IR.Type}, broadcast_dimensions=nothing, comparison_direction, compare_type=nothing, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[lhs, rhs, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[namedattribute("comparison_direction", comparison_direction), ]
    !isnothing(result_0) && push!(op_ty_results, result_0)
    !isnothing(broadcast_dimensions) && push!(attributes, namedattribute("broadcast_dimensions", broadcast_dimensions))
    !isnothing(compare_type) && push!(attributes, namedattribute("compare_type", compare_type))
    
    create_operation(
        "chlo.broadcast_compare", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`broadcast_complex`

Performs element-wise conversion of a pair of real and imaginary values to
a complex value.
"""
function broadcast_complex(lhs::Value, rhs::Value; result_0=nothing::Union{Nothing, IR.Type}, broadcast_dimensions=nothing, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[lhs, rhs, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result_0) && push!(op_ty_results, result_0)
    !isnothing(broadcast_dimensions) && push!(attributes, namedattribute("broadcast_dimensions", broadcast_dimensions))
    
    create_operation(
        "chlo.broadcast_complex", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`broadcast_divide`

Returns `lhs / rhs` element-wise.

See
https://www.tensorflow.org/xla/operation_semantics#element-wise_binary_arithmetic_operations.
"""
function broadcast_divide(lhs::Value, rhs::Value; result_0=nothing::Union{Nothing, IR.Type}, broadcast_dimensions=nothing, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[lhs, rhs, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result_0) && push!(op_ty_results, result_0)
    !isnothing(broadcast_dimensions) && push!(attributes, namedattribute("broadcast_dimensions", broadcast_dimensions))
    
    create_operation(
        "chlo.broadcast_divide", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`broadcast_maximum`

Returns `max(lhs, rhs)` element-wise.

See
https://www.tensorflow.org/xla/operation_semantics#element-wise_binary_arithmetic_operations.
"""
function broadcast_maximum(lhs::Value, rhs::Value; result_0=nothing::Union{Nothing, IR.Type}, broadcast_dimensions=nothing, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[lhs, rhs, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result_0) && push!(op_ty_results, result_0)
    !isnothing(broadcast_dimensions) && push!(attributes, namedattribute("broadcast_dimensions", broadcast_dimensions))
    
    create_operation(
        "chlo.broadcast_maximum", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`broadcast_minimum`

Returns `min(lhs, rhs)` element-wise.

See
https://www.tensorflow.org/xla/operation_semantics#element-wise_binary_arithmetic_operations.
"""
function broadcast_minimum(lhs::Value, rhs::Value; result_0=nothing::Union{Nothing, IR.Type}, broadcast_dimensions=nothing, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[lhs, rhs, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result_0) && push!(op_ty_results, result_0)
    !isnothing(broadcast_dimensions) && push!(attributes, namedattribute("broadcast_dimensions", broadcast_dimensions))
    
    create_operation(
        "chlo.broadcast_minimum", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`broadcast_multiply`

Returns `lhs * rhs` element-wise.

See
https://www.tensorflow.org/xla/operation_semantics#element-wise_binary_arithmetic_operations.
"""
function broadcast_multiply(lhs::Value, rhs::Value; result_0=nothing::Union{Nothing, IR.Type}, broadcast_dimensions=nothing, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[lhs, rhs, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result_0) && push!(op_ty_results, result_0)
    !isnothing(broadcast_dimensions) && push!(attributes, namedattribute("broadcast_dimensions", broadcast_dimensions))
    
    create_operation(
        "chlo.broadcast_multiply", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`broadcast_next_after`

Returns the next representable value of `lhs` in the direction of `rhs`,
element-wise. It can also return a subnormal number.

Equivalent to the C++ std::nextafter function.
"""
function broadcast_next_after(lhs::Value, rhs::Value; result_0=nothing::Union{Nothing, IR.Type}, broadcast_dimensions=nothing, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[lhs, rhs, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result_0) && push!(op_ty_results, result_0)
    !isnothing(broadcast_dimensions) && push!(attributes, namedattribute("broadcast_dimensions", broadcast_dimensions))
    
    create_operation(
        "chlo.broadcast_next_after", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`broadcast_or`

Returns `logical_or(lhs, rhs)` element-wise.

See
https://www.tensorflow.org/xla/operation_semantics#element-wise_binary_arithmetic_operations.
"""
function broadcast_or(lhs::Value, rhs::Value; result_0=nothing::Union{Nothing, IR.Type}, broadcast_dimensions=nothing, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[lhs, rhs, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result_0) && push!(op_ty_results, result_0)
    !isnothing(broadcast_dimensions) && push!(attributes, namedattribute("broadcast_dimensions", broadcast_dimensions))
    
    create_operation(
        "chlo.broadcast_or", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`broadcast_polygamma`

Returns `Polygamma(operand, operand)` element-wise.
"""
function broadcast_polygamma(lhs::Value, rhs::Value; result_0=nothing::Union{Nothing, IR.Type}, broadcast_dimensions=nothing, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[lhs, rhs, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result_0) && push!(op_ty_results, result_0)
    !isnothing(broadcast_dimensions) && push!(attributes, namedattribute("broadcast_dimensions", broadcast_dimensions))
    
    create_operation(
        "chlo.broadcast_polygamma", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`broadcast_power`

Returns `lhs ^ rhs` element-wise.

See
https://www.tensorflow.org/xla/operation_semantics#element-wise_binary_arithmetic_operations.
"""
function broadcast_power(lhs::Value, rhs::Value; result_0=nothing::Union{Nothing, IR.Type}, broadcast_dimensions=nothing, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[lhs, rhs, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result_0) && push!(op_ty_results, result_0)
    !isnothing(broadcast_dimensions) && push!(attributes, namedattribute("broadcast_dimensions", broadcast_dimensions))
    
    create_operation(
        "chlo.broadcast_power", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`broadcast_remainder`

Returns `lhs % rhs` element-wise.

See
https://www.tensorflow.org/xla/operation_semantics#element-wise_binary_arithmetic_operations.
"""
function broadcast_remainder(lhs::Value, rhs::Value; result_0=nothing::Union{Nothing, IR.Type}, broadcast_dimensions=nothing, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[lhs, rhs, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result_0) && push!(op_ty_results, result_0)
    !isnothing(broadcast_dimensions) && push!(attributes, namedattribute("broadcast_dimensions", broadcast_dimensions))
    
    create_operation(
        "chlo.broadcast_remainder", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`broadcast_select`

Constructs an output array from elements of two input arrays, based on the
values of a predicate array.

See https://www.tensorflow.org/xla/operation_semantics#select
"""
function broadcast_select(pred::Value, on_true::Value, on_false::Value; result_0=nothing::Union{Nothing, IR.Type}, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[pred, on_true, on_false, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result_0) && push!(op_ty_results, result_0)
    
    create_operation(
        "chlo.broadcast_select", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`broadcast_shift_left`

Returns `lhs << rhs` element-wise.

See
https://www.tensorflow.org/xla/operation_semantics#element-wise_binary_arithmetic_operations.
"""
function broadcast_shift_left(lhs::Value, rhs::Value; result_0=nothing::Union{Nothing, IR.Type}, broadcast_dimensions=nothing, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[lhs, rhs, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result_0) && push!(op_ty_results, result_0)
    !isnothing(broadcast_dimensions) && push!(attributes, namedattribute("broadcast_dimensions", broadcast_dimensions))
    
    create_operation(
        "chlo.broadcast_shift_left", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`broadcast_shift_right_arithmetic`

Returns `lhs >> rhs` element-wise.

See
https://www.tensorflow.org/xla/operation_semantics#element-wise_binary_arithmetic_operations.
"""
function broadcast_shift_right_arithmetic(lhs::Value, rhs::Value; result_0=nothing::Union{Nothing, IR.Type}, broadcast_dimensions=nothing, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[lhs, rhs, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result_0) && push!(op_ty_results, result_0)
    !isnothing(broadcast_dimensions) && push!(attributes, namedattribute("broadcast_dimensions", broadcast_dimensions))
    
    create_operation(
        "chlo.broadcast_shift_right_arithmetic", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`broadcast_shift_right_logical`

Returns `lhs >> rhs` element-wise.

See
https://www.tensorflow.org/xla/operation_semantics#element-wise_binary_arithmetic_operations.
"""
function broadcast_shift_right_logical(lhs::Value, rhs::Value; result_0=nothing::Union{Nothing, IR.Type}, broadcast_dimensions=nothing, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[lhs, rhs, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result_0) && push!(op_ty_results, result_0)
    !isnothing(broadcast_dimensions) && push!(attributes, namedattribute("broadcast_dimensions", broadcast_dimensions))
    
    create_operation(
        "chlo.broadcast_shift_right_logical", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`broadcast_subtract`

Returns `lhs - rhs` element-wise.

See
https://www.tensorflow.org/xla/operation_semantics#element-wise_binary_arithmetic_operations.
"""
function broadcast_subtract(lhs::Value, rhs::Value; result_0=nothing::Union{Nothing, IR.Type}, broadcast_dimensions=nothing, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[lhs, rhs, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result_0) && push!(op_ty_results, result_0)
    !isnothing(broadcast_dimensions) && push!(attributes, namedattribute("broadcast_dimensions", broadcast_dimensions))
    
    create_operation(
        "chlo.broadcast_subtract", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`broadcast_xor`

Returns `logical_xor(lhs, rhs)` element-wise.

See
https://www.tensorflow.org/xla/operation_semantics#element-wise_binary_arithmetic_operations.
"""
function broadcast_xor(lhs::Value, rhs::Value; result_0=nothing::Union{Nothing, IR.Type}, broadcast_dimensions=nothing, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[lhs, rhs, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result_0) && push!(op_ty_results, result_0)
    !isnothing(broadcast_dimensions) && push!(attributes, namedattribute("broadcast_dimensions", broadcast_dimensions))
    
    create_operation(
        "chlo.broadcast_xor", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`broadcast_zeta`

Returns `Zeta(operand, operand)` element-wise.

\$\$
\\(\\zeta(x, q) = \\sum_{n=0}^{\\infty} (q + n)^{-x}\\)
\$\$
"""
function broadcast_zeta(lhs::Value, rhs::Value; result_0=nothing::Union{Nothing, IR.Type}, broadcast_dimensions=nothing, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[lhs, rhs, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result_0) && push!(op_ty_results, result_0)
    !isnothing(broadcast_dimensions) && push!(attributes, namedattribute("broadcast_dimensions", broadcast_dimensions))
    
    create_operation(
        "chlo.broadcast_zeta", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`conj`

Returns `Conj(operand)` element-wise.

\$\$
\\conj(x) = (\\real(x), \\neg(\\imag(x)))
\$\$
"""
function conj(operand::Value; result=nothing::Union{Nothing, IR.Type}, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[operand, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result) && push!(op_ty_results, result)
    
    create_operation(
        "chlo.conj", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`constant_like`

Returns a splat constant of the same shape as the operand.
"""
function constant_like(operand::Value; result_0=nothing::Union{Nothing, IR.Type}, value, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[operand, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[namedattribute("value", value), ]
    !isnothing(result_0) && push!(op_ty_results, result_0)
    
    create_operation(
        "chlo.constant_like", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`constant`

Represents a constant value.
"""
function constant(; output=nothing::Union{Nothing, IR.Type}, value, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[namedattribute("value", value), ]
    !isnothing(output) && push!(op_ty_results, output)
    
    create_operation(
        "chlo.constant", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`cosh`

Returns `Cosh(operand)` element-wise.

\$\$
\\cosh(x) = (e^x + e^-x) / 2
\$\$
"""
function cosh(operand::Value; result=nothing::Union{Nothing, IR.Type}, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[operand, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result) && push!(op_ty_results, result)
    
    create_operation(
        "chlo.cosh", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`digamma`

Returns `Digamma(operand)` element-wise.
"""
function digamma(operand::Value; result=nothing::Union{Nothing, IR.Type}, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[operand, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result) && push!(op_ty_results, result)
    
    create_operation(
        "chlo.digamma", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`dynamic_reshape`

Reshapes `operand` to `output_shape`. This allows a single dimension to be
specified as -1, and it will be computed by the op to create a valid
reshape.

Requires:
- The length of `output_shape` is equal to the rank of `result`.
- The number of elements in `operand` (that is, the product of extents of
  its shape) is equal to the number of elements in `output_shape` (that is,
  the product of values in `output_shape` with one dimension possibly
  computed).
- All shape values should be at least -1, and only one extent can be -1.
"""
function dynamic_reshape(operand::Value, output_shape::Value; result::IR.Type, location=Location())
    op_ty_results = IR.Type[result, ]
    operands = Value[operand, output_shape, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    
    create_operation(
        "chlo.dynamic_reshape", location;
        operands, owned_regions, successors, attributes,
        results=op_ty_results,
        result_inference=false
    )
end

"""
`erf_inv`

Returns `ErfInv(operand)` element-wise.
"""
function erf_inv(operand::Value; result=nothing::Union{Nothing, IR.Type}, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[operand, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result) && push!(op_ty_results, result)
    
    create_operation(
        "chlo.erf_inv", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`erf`

Computes the Gauss error function of `x` element-wise.

erf(x) = erf_impl(x)            if |x| < 1
       = 1 - erfc_impl(x)       otherwise
"""
function erf(operand::Value; result=nothing::Union{Nothing, IR.Type}, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[operand, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result) && push!(op_ty_results, result)
    
    create_operation(
        "chlo.erf", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`erfc`

Computes an approximation of the error function complement (1 - erf(x)).

erfc(x) = erfc_impl(x)           if |x| > 1
        = 1 - erf_impl(x)        otherwise
"""
function erfc(operand::Value; result=nothing::Union{Nothing, IR.Type}, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[operand, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result) && push!(op_ty_results, result)
    
    create_operation(
        "chlo.erfc", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`is_inf`

Returns if a value is +/-inf element-wise.
"""
function is_inf(operand::Value; result=nothing::Union{Nothing, IR.Type}, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[operand, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result) && push!(op_ty_results, result)
    
    create_operation(
        "chlo.is_inf", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`is_neg_inf`

Returns if a value is -inf element-wise.
"""
function is_neg_inf(operand::Value; result=nothing::Union{Nothing, IR.Type}, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[operand, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result) && push!(op_ty_results, result)
    
    create_operation(
        "chlo.is_neg_inf", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`is_pos_inf`

Returns if a value is +inf element-wise.
"""
function is_pos_inf(operand::Value; result=nothing::Union{Nothing, IR.Type}, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[operand, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result) && push!(op_ty_results, result)
    
    create_operation(
        "chlo.is_pos_inf", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`lgamma`

Returns `Lgamma(operand)` element-wise.
"""
function lgamma(operand::Value; result=nothing::Union{Nothing, IR.Type}, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[operand, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result) && push!(op_ty_results, result)
    
    create_operation(
        "chlo.lgamma", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`minimum_broadcast_shapes`

Given two or more 1D tensors representing shapes, returns one 1D tensor for
each operand, where operand `i` corresponds to output `i`.

The returned tensors have the property that they specify a shape which is a
reshape of the corresponding input shape, and the broadcasted output shape
(using shape::BroadcastOp) of the returned shapes is a reshape of the
broadcasted output shape of the input shapes. Among all possibilities with
this property, the one is chosen which minimizes the rank of each returned
shape.

The general idea of this op is that it can be used for ops which have a
broadcasting semantic to operate on shapes with a possibly smaller rank
while preserving equivalence of the computed values. After computing the
result of the op using reshaped operands, the result can be reshaped to the
result that would have been originally computed.

Here is an example with two input shapes:

```mlir
chlo.minimum_broadcast_shapes [1, 2, 3, 1, 2, 1],
                                 [1, 1, 1, 2, 3] -> [6, 2, 1], [2, 3]
```

The broadcasted output shape of the operands is [1, 2, 3, 1, 2, 3], the
broadcasted output shape of the outputs is [6, 2, 3]. These two shapes are
reshapes of each other, and also each output is a reshape of the
corresponding input.
"""
function minimum_broadcast_shapes(shapes::Vector{Value}; results::Vector{IR.Type}, location=Location())
    op_ty_results = IR.Type[results..., ]
    operands = Value[shapes..., ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    
    create_operation(
        "chlo.minimum_broadcast_shapes", location;
        operands, owned_regions, successors, attributes,
        results=op_ty_results,
        result_inference=false
    )
end

"""
`next_after`

Returns the next representable value of `x` in the direction of `y`,
element-wise. It can also return a subnormal number.

Equivalent to the C++ std::nextafter function.
"""
function next_after(x::Value, y::Value; result=nothing::Union{Nothing, IR.Type}, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[x, y, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result) && push!(op_ty_results, result)
    
    create_operation(
        "chlo.next_after", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`polygamma`

Returns `Polygamma(operand, operand)` element-wise.
"""
function polygamma(n::Value, x::Value; result=nothing::Union{Nothing, IR.Type}, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[n, x, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result) && push!(op_ty_results, result)
    
    create_operation(
        "chlo.polygamma", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`sinh`

Returns `Sinh(operand)` element-wise.

\$\$
\\sinh(x) = (e^x - e^-x) / 2                     if |x| < 1
         = e^(x + log(1/2)) - e^(-x + log(1/2)) otherwise.
\$\$
"""
function sinh(operand::Value; result=nothing::Union{Nothing, IR.Type}, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[operand, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result) && push!(op_ty_results, result)
    
    create_operation(
        "chlo.sinh", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`tan`

Returns `Tan(operand)` element-wise.

\$\$
\\tan(x) = \\sin(x) / \\cos(x)
\$\$
"""
function tan(operand::Value; result=nothing::Union{Nothing, IR.Type}, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[operand, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result) && push!(op_ty_results, result)
    
    create_operation(
        "chlo.tan", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`top_k`

If the input is a vector (rank-1), finds the `k` largest entries in the vector
and outputs their values and indices as vectors.  Thus `values[j]` is the
`j`-th largest entry in `input`, and its index is `indices[j]`.

For matrices (resp. higher rank input), computes the top `k` entries in each
row (resp. vector along the last dimension).  Thus,

    values.shape = indices.shape = input.shape[:-1] + [k]

If two elements are equal, the lower-index element appears first.
"""
function top_k(operand::Value; values=nothing::Union{Nothing, IR.Type}, indices=nothing::Union{Nothing, IR.Type}, k, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[operand, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[namedattribute("k", k), ]
    !isnothing(values) && push!(op_ty_results, values)
    !isnothing(indices) && push!(op_ty_results, indices)
    
    create_operation(
        "chlo.top_k", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

"""
`zeta`

Returns `Zeta(operand, operand)` element-wise.

\$\$
\\(\\zeta(x, q) = \\sum_{n=0}^{\\infty} (q + n)^{-x}\\)
\$\$
"""
function zeta(x::Value, q::Value; result=nothing::Union{Nothing, IR.Type}, location=Location())
    op_ty_results = IR.Type[]
    operands = Value[x, q, ]
    owned_regions = Region[]
    successors = Block[]
    attributes = NamedAttribute[]
    !isnothing(result) && push!(op_ty_results, result)
    
    create_operation(
        "chlo.zeta", location;
        operands, owned_regions, successors, attributes,
        results=(length(op_ty_results) == 0 ? nothing : op_ty_results),
        result_inference=(length(op_ty_results) == 0 ? true : false)
    )
end

end # chlo
